/*
 *  Calculation of Pi using quadrature realized with a fork/join approach using an actor system.
 * 
 *  Copyright © 2009 Russel Winder
 */

import scala.actors.Actor

object Pi_Scala_Actors extends Application {
  def execute ( numberOfThreads : Int ) {
    val n = 1000000000l
    val delta = 1.0 / n
    val startTimeNanos = System.nanoTime ( )
    val sliceSize = n / numberOfThreads
    val calculators = new Array[Actor] ( numberOfThreads )
    val accumulator = new Actor ( ) {
      def act ( ) {
        var sum = 0.0
        calculators.foreach ( ( calculator ) => receive { case d => sum += d.asInstanceOf[Double] } )
        val pi = 4.0 * sum * delta
        val elapseTime = ( System.nanoTime ( ) - startTimeNanos ) / 1e9
        println ( "==== Scala Actors pi = " + pi )
        println ( "==== Scala Actors iteration count = " + n )
        println ( "==== Scala Actors elapse = " + elapseTime )
        println ( "==== Scala Actors processor count = " + Runtime.getRuntime ( ).availableProcessors ( ) )
        println ( "==== Scala Actors threads count = " + numberOfThreads )
        sequencer ! 0
      }
    }
    accumulator.start ( )
    for ( i <- 0 until numberOfThreads ) {
      calculators ( i ) = Actor.actor {
        var j = 1 + i * sliceSize
        val end = ( i + 1 ) * sliceSize 
        var sum = 0.0
        while ( j <=  end ) {
          val x = ( j - 0.5 ) * delta
          sum += 1.0 / ( 1.0 + x * x )
          j += 1
        }
        accumulator ! sum
      }
    }
  }
  val sequencer = new Actor ( ) {
    def act ( ) {
      execute ( 1 )
      receive { case d => }
      println ( )
      execute ( 2 )
      receive { case d => }
      println ( )
      execute ( 8 )
      receive { case d => }
      println ( )
      execute ( 32 )
    }
  }
  sequencer.start ( )
}
