/*
 *  Calculation of Pi using quadrature realized with futures.
 *
 *  Copyright © 2009 Russel Winder
 */

import scala.actors.Actor
import scala.actors.Future
import scala.actors.Futures

object Pi_Scala_Futures extends Application {
  def execute ( numberOfThreads : Int ) {
    val n = 1000000000l
    val delta = 1.0 / n
    val startTimeNanos = System.nanoTime ( )
    val sliceSize = n / numberOfThreads
    val partialSums = new Array[Future[Double]] ( numberOfThreads )
    for ( i <- 0 until numberOfThreads ) {
      partialSums ( i ) = Futures.future {
        var j = 1 + i * sliceSize
        val end = ( i + 1 ) * sliceSize 
        var sum = 0.0
        while ( j <=  end ) {
          val x = ( j - 0.5 ) * delta
          sum += 1.0 / ( 1.0 + x * x )
          j += 1
        }
        sum
      }
    }
    var sum = 0.0
    for ( p <- partialSums ) { sum += p ( ) }
    val pi = 4.0 * sum * delta
    val elapseTime = ( System.nanoTime ( ) - startTimeNanos ) / 1e9
    println ( "==== Scala Futures pi = " + pi )
    println ( "==== Scala Futures iteration count = " + n )
    println ( "==== Scala Futures elapse = " + elapseTime )
    println ( "==== Scala Futures processor count = " + Runtime.getRuntime ( ).availableProcessors ( ) )
    println ( "==== Scala Futures threads count = " + numberOfThreads )
  }
  execute ( 1 )
  println ( )
  execute ( 2 )
  println ( )
  execute ( 8 )
  println ( )
  execute ( 32 )
}
