/*
 *  A C++ program to calculate Pi using quadrature as an OpenMP annotated algorithm.
 *
 *  Copyright © 2008-9 Russel Winder
 */

#include <iostream>
#include <iomanip>
#include <omp.h>
#include "microsecondTime.h"

int main ( ) {
  const long n = 1000000000l ;
  const long double delta = 1.0 / n ;
  const long long startTimeMicros = microsecondTime ( ) ;
  long double sum = 0.0 ;
#pragma omp parallel for reduction ( + : sum )
  for ( long i = 1 ; i <= n ; ++i ) {
    const long double x = ( i - 0.5 ) * delta ;
    sum += 1.0 / ( 1.0 + x * x ) ;
  }
  const long double pi = 4.0 * sum * delta ;
  const long double elapseTime = ( microsecondTime ( ) - startTimeMicros ) / 1e6 ;
  std::cout << "==== C++ OpenMP Implicit pi = " << std::setprecision ( 25 ) << pi << std::endl ;
  std::cout << "==== C++ OpenMP Implicit iteration count = " << n << std::endl ;
  std::cout << "==== C++ OpenMP Implicit elapse = " << elapseTime << std::endl ;
  std::cout << "==== C++ OpenMP Implicit processor count = " << omp_get_num_procs ( ) << std::endl ;
  return 0 ;
}
