#! /usr/bin/env python
# -*- mode: python; coding: utf-8 -*-

#  Calculation of Pi using quadrature. Using  the processing package to provide a process pool.
#
#  Copyright © 2008-9 Russel Winder

import time
import processing

def processSlice ( start , end , delta ) :
    sum = 0.0
    #  NB Using "for i in range ( start , end + 1 )" here would be a very bad move since range creates a
    #  list and with numbers this big you really don't want to do that.
    i = start
    while i < end + 1 :
        x = ( i - 0.5 ) * delta
        sum += 1.0 / ( 1.0 + x * x )
        i += 1
    return sum

n = 10000000 # 100 times fewer due to speed issues.
delta = 1.0 / n
startTime = time.time ( ) # time.clock ( )
nProcessors = processing.cpuCount ( )
slice = n / nProcessors
pool = processing.Pool ( processes = nProcessors )
results = [ pool.apply_async ( processSlice , args = ( 1 + i * slice , ( i + 1 ) * slice , delta ) ) for i in range ( 0 , nProcessors ) ]
pool.close ( )
pool.join ( )
results = [ item.get ( ) for item in results ]
pi = 4.0 * sum ( results ) * delta
elapseTime = time.time ( ) - startTime # time.clock ( ) - startTime # Using clock gives a value close to 0 always!
print "==== Python Processing Pool pi =" , pi
print "==== Python Processing Pool iteration count =", n
print "==== Python Processing Pool elapse =" , elapseTime
print "==== Python Processing Pool processor count =" , nProcessors
